// This file is part of the disk cataloger Wssindex, but was written by a
// 3rd party.  See comments below for author and licensing information.
// Revised to not use wsprintf

// Eject media code for Win/NT/2K/XP from
//	http://support.microsoft.com/default.aspx?scid=kb;en-us;165721
// This link is broken and I can't find an alternate source.

#include <windows.h>
#include <winioctl.h>

BOOL ejectNT(TCHAR cDriveLetter);

HANDLE OpenVolume(TCHAR cDriveLetter);
BOOL LockVolume(HANDLE hVolume);
BOOL DismountVolume(HANDLE hVolume);
BOOL PreventRemovalOfVolume(HANDLE hVolume, BOOL fPrevent);
BOOL AutoEjectVolume(HANDLE hVolume);
BOOL CloseVolume(HANDLE hVolume);

//LPTSTR szVolumeFormat = (TCHAR *)TEXT("\\\\.\\%c:");
//LPTSTR szRootFormat = (TCHAR *)TEXT("%c:\\");

HANDLE OpenVolume(TCHAR cDriveLetter)
	{
	HANDLE hVolume;
	UINT uDriveType;
//	TCHAR szVolumeName[8];
//	TCHAR szRootName[5];
	TCHAR szVolumeName[8] = TEXT("\\\\.\\X:");
	TCHAR szRootName[5] = TEXT("X:\\");
	DWORD dwAccessFlags;

//	wsprintf(szRootName, szRootFormat, cDriveLetter);
	szRootName[0] = cDriveLetter;

	uDriveType = GetDriveType(szRootName);
	switch(uDriveType)
		{
		case DRIVE_REMOVABLE:
			dwAccessFlags = GENERIC_READ | GENERIC_WRITE;
			break;
		case DRIVE_CDROM:
			dwAccessFlags = GENERIC_READ;
			break;
		default:
			return INVALID_HANDLE_VALUE;
		}

//	wsprintf(szVolumeName, szVolumeFormat, cDriveLetter);
	szVolumeName[4] = cDriveLetter;

	hVolume = CreateFile( szVolumeName,
								 dwAccessFlags,
								 FILE_SHARE_READ | FILE_SHARE_WRITE,
								 nullptr,
								 OPEN_EXISTING,
								 0,
								 nullptr );

	return hVolume;
	}

BOOL CloseVolume(HANDLE hVolume)
	{
	return CloseHandle(hVolume);
	}

#define LOCK_TIMEOUT		10000		// 10 Seconds
#define LOCK_RETRIES		20

BOOL LockVolume(HANDLE hVolume)
	{
	DWORD dwBytesReturned;
	DWORD dwSleepAmount;
	int nTryCount;

	dwSleepAmount = LOCK_TIMEOUT / LOCK_RETRIES;

// Do this in a loop until a timeout period has expired
	for (nTryCount = 0; nTryCount < LOCK_RETRIES; nTryCount++)
		{
		if (DeviceIoControl(	hVolume,
									FSCTL_LOCK_VOLUME,
									nullptr, 0,
									nullptr, 0,
									&dwBytesReturned,
									nullptr))
			return true;

		Sleep(dwSleepAmount);
		}

	return false;
	}

BOOL DismountVolume(HANDLE hVolume)
	{
	DWORD dwBytesReturned;

	return DeviceIoControl( hVolume,
									FSCTL_DISMOUNT_VOLUME,
									nullptr, 0,
									nullptr, 0,
									&dwBytesReturned,
									nullptr);
	}

BOOL PreventRemovalOfVolume(HANDLE hVolume, BOOL fPreventRemoval)
	{
	DWORD dwBytesReturned;
	PREVENT_MEDIA_REMOVAL PMRBuffer;

	PMRBuffer.PreventMediaRemoval = static_cast<unsigned char>(fPreventRemoval);

	return DeviceIoControl( hVolume,
									IOCTL_STORAGE_MEDIA_REMOVAL,
									&PMRBuffer, sizeof(PREVENT_MEDIA_REMOVAL),
									nullptr, 0,
									&dwBytesReturned,
									nullptr);
	}

BOOL AutoEjectVolume(HANDLE hVolume)
	{
	DWORD dwBytesReturned;

	return DeviceIoControl( hVolume,
									IOCTL_STORAGE_EJECT_MEDIA,
									nullptr, 0,
									nullptr, 0,
									&dwBytesReturned,
									nullptr);
	}

BOOL ejectNT(TCHAR cDriveLetter)
	{
	HANDLE hVolume;

// Open the volume.
	hVolume = OpenVolume(cDriveLetter);
	if (hVolume == INVALID_HANDLE_VALUE)
		return false;

// Lock and dismount the volume.
	LockVolume(hVolume);
	DismountVolume(hVolume);

// Set prevent removal to false and eject the volume.
	PreventRemovalOfVolume(hVolume, false);
	AutoEjectVolume(hVolume);

// Close the volume so other processes can use the drive.
	if (!CloseVolume(hVolume))
		return false;

	return true;
	}
