// This file is part of the disk cataloger Wssindex.  Wssindex is copyright
// 1985-2021 by Robert W. Babcock and WSS Division of DDC.  Effective 2010,
// Wssindex is distributed under the terms of the GNU General Public
// License.  Contact the author at wssddc@wssddc.com or see the project web
// page at <https://www.wssddc.com/wssindex/>.
//
// Wssindex uses the open source Qt toolkit from the Qt Project (qt-project.org)
// and The Qt Company (www.qt.io).
//
// Wssindex is free software: you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by the
// Free Software Foundation, either version 3 of the License, or (at your
// option) any later version.
//
// Wssindex is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
// for more details.
//
// You should have received a copy of the GNU General Public License along
// with Wssindex.  If not, see <https://www.gnu.org/licenses/>.

#include "OtherWssi/wssindex.h"
#include "Dialogs/mainwindow.h"
#include "ui_mainwindow.h"
#include "Dialogs/fullsortdialog.h"
#include "Dialogs/simplesortdialog.h"
#include "OtherWssi/tablelayout.h"
#include "OtherWssi/wssiproto.h"
#include <QtDebug>

void MainWindow::slot_closeSorts()
	{
	Q_EMIT signal_close_sorts();
	}

void MainWindow::slot_pushButtonFullSortClicked()
	{
	FullSortDialog *f = new FullSortDialog(this);
	f->setAttribute(Qt::WA_DeleteOnClose);
	f->show();
	}

void MainWindow::slot_pushButtonSortClicked()
	{
	SimpleSortDialog *s = new SimpleSortDialog(this);
	s->setAttribute(Qt::WA_DeleteOnClose);
	s->show();
	}

// Sort with configured number of threads
void MainWindow::slot_sortdatabase()
	{
	sortdatabase(0);
	}

// Force the number of threads to use when sorting
void MainWindow::slot_sortforcedatabase(int force)
	{
	doSort(Sortindx, Noselect, wssicomp, force);
	}

void MainWindow::sortdatabase(int force)
	{
	doSort(Sortindx, Noselect, wssicomp, force);
	slot_updateDisplay(true);

	int fieldindex;
	switch(SortKey[0])
		{
		case SortKey_FN:		fieldindex = 0; break;
		case SortKey_EXT:		fieldindex = 1; break;
		case SortKey_DISK:	fieldindex = 2; break;
		case SortKey_SUB:		fieldindex = 3; break;
		case SortKey_ARCNAM: fieldindex = 4; break;
		case SortKey_SIZE:	fieldindex = 5; break;
		case SortKey_TTAG:	fieldindex = 6; break;
		case SortKey_CAT:		fieldindex = 7; break;
		case SortKey_COM:		fieldindex = 8; break;
		case SortKey_EXTRA:	fieldindex = 9; break;
		default:
			return;
		}
	if(!fieldlayout[fieldindex].enabled)
		return;
	if(fieldlayout[fieldindex].row > 0)
		return;
	int column = fieldlayout[fieldindex].column;
	Qt::SortOrder indicator;
	indicator = Sortdir[0]>0 ? Qt::AscendingOrder : Qt::DescendingOrder;
	ui->tableView->horizontalHeader()->setSortIndicatorShown(true);
	ui->tableView->horizontalHeader()->setSortIndicator(column, indicator);
	}

void MainWindow::slot_tableClicked(int column)
	{
	int localsortfield, localsortdir;
// Field to be sorted is probably row 1 in column, but could
// be latter row if there is a blank cell.
	int localsortrow = 99;
	localsortfield = 0;		// This is just to quiet compiler warning
	for(int i=0; i<NFIELDS; i++)
		{
		if(!fieldlayout[i].enabled)
			continue;
		if(column != fieldlayout[i].column)
			continue;
		if(fieldlayout[i].row < localsortrow)
			{
			localsortrow = fieldlayout[i].row;
			localsortfield = i;
			}
		}
	if(headerSortDir[localsortfield] > 0)
		localsortdir = -1;
	else
		localsortdir = 1;
	headerSortDir[localsortfield] = localsortdir;
	Nkey = 1;
	Sortdir[0] = localsortdir;
	switch(localsortfield)
		{
		case 0:	// Filename (long) and extension
			SortKey[0] = SortKey_FN;
			SortKey[1] = SortKey_EXT;
			Sortdir[1] = localsortdir;
			Nkey = 2;
			break;

		case 1:	// Extension
			SortKey[0] = SortKey_EXT;
			break;

		case 2:	// Volume
			SortKey[0] = SortKey_DISK;
			break;

		case 3:	// Subdirectory
			SortKey[0] = SortKey_SUB;
			break;

		case 4:	// Archive
			SortKey[0] = SortKey_ARCNAM;
			SortKey[1] = SortKey_ARCEXT;
			Sortdir[1] = localsortdir;
			Nkey = 2;
			break;

		case 5:	// Size
			SortKey[0] = SortKey_SIZE;
			break;

		case 6:	// Timestamp
			SortKey[0] = SortKey_TTAG;
			break;

		case 7:	// Category
			SortKey[0] = SortKey_CAT;
			break;

		case 8:	// Comment
			SortKey[0] = SortKey_COM;
			break;

		case 9:	// Extra info
			SortKey[0] = SortKey_EXTRA;
			break;

		default:	// Shouldn't happen
			return;
		}
	slot_sortdatabase();
	}

// Sortindx elements to be removed have been set to Nofile, i.e. larger than database
void MainWindow::sortListPurge(void)
	{
	unsigned long k = 0;
	for(unsigned long i=0; i<Noselect; i++)
		{
		if(Sortindx[i] == Nofile)
			continue;
		Sortindx[k] = Sortindx[i];
		k++;
		}

	Noselect = k;
	slot_updateDisplay(true);
	}
