// This file is part of the disk cataloger Wssindex.  Wssindex is copyright
// 1985-2021 by Robert W. Babcock and WSS Division of DDC.  Effective 2010,
// Wssindex is distributed under the terms of the GNU General Public
// License.  Contact the author at wssddc@wssddc.com or see the project web
// page at <https://www.wssddc.com/wssindex/>.
//
// Wssindex uses the open source Qt toolkit from the Qt Project (qt-project.org)
// and The Qt Company (www.qt.io).
//
// Wssindex is free software: you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by the
// Free Software Foundation, either version 3 of the License, or (at your
// option) any later version.
//
// Wssindex is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
// for more details.
//
// You should have received a copy of the GNU General Public License along
// with Wssindex.  If not, see <https://www.gnu.org/licenses/>.

#ifndef NAMESQUERYDIALOG_H
#define NAMESQUERYDIALOG_H

#include <QMainWindow>

namespace Ui
	{
	class NamesQueryDialog;
	}

class MainWindow;

class NamesQueryDialog : public QMainWindow
	{
	Q_OBJECT

public:
	explicit NamesQueryDialog(QWidget *parent);
	~NamesQueryDialog() override;

private:
	void restoreSettings();
	void keyPressEvent(QKeyEvent * event) override;
	void wheelEvent(QWheelEvent *event) override;

private:
	Ui::NamesQueryDialog *nq;

protected:
	void closeEvent(QCloseEvent *event) override;

Q_SIGNALS:
	void signal_updateDisplay(bool);
	void signal_keyToTable(int, int);
	void signal_wheelToTable(QPoint, int, Qt::MouseButtons,
										 Qt::KeyboardModifiers, Qt::Orientation);

public Q_SLOTS:
	void slot_diskListChanged();
	void slot_updateNoId(bool checked);

private Q_SLOTS:
	void slot_comboBoxVolsChanged();
	void slot_lineEditCategoryChanged();
	void slot_lineEditCommentChanged();
	void slot_lineEditExtraChanged();
	void slot_lineEditFilenameChanged();
	void slot_pushButtonCloseClicked();
	void slot_pushButtonQueryClicked();
	void slot_radioButtonCategoryToggled(bool);
	void slot_radioButtonCommentToggled(bool);
	void slot_radioButtonExtraToggled(bool);
	void slot_radioButtonDiskToggled(bool);
	void slot_radioButtonFilenameToggled(bool);
	void slot_radioButtonSubdirChanged();
	void slot_radioButtonSubdirToggled(bool);
	};
#endif // NAMESQUERYDIALOG_H
