// This file is part of the disk cataloger Wssindex.  Wssindex is copyright
// 1985-2021 by Robert W. Babcock and WSS Division of DDC.  Effective 2010,
// Wssindex is distributed under the terms of the GNU General Public
// License.  Contact the author at wssddc@wssddc.com or see the project web
// page at <https://www.wssddc.com/wssindex/>.
//
// Wssindex uses the open source Qt toolkit from the Qt Project (qt-project.org)
// and The Qt Company (www.qt.io).
//
// Wssindex is free software: you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by the
// Free Software Foundation, either version 3 of the License, or (at your
// option) any later version.
//
// Wssindex is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
// for more details.
//
// You should have received a copy of the GNU General Public License along
// with Wssindex.  If not, see <https://www.gnu.org/licenses/>.

#ifndef TABLEPREVIEW_H
#define TABLEPREVIEW_H

#include "Dialogs/mainwindow.h"

#include <QMainWindow>
#include <QTableWidget>

namespace Ui
	{
	class TablePreview;
	}

class TableDesignDialog;

class TablePreview : public QMainWindow
	{
	Q_OBJECT

public:
	explicit TablePreview(MainWindow *parent, TableDesignDialog *designparent = nullptr);
	~TablePreview() override;
	void set_preview_update_enabled(bool e) {preview_update_enabled = e;}

protected:
	void closeEvent(QCloseEvent *event) override;

private:
	void restoreSettings();
	void keyPressEvent(QKeyEvent * event) override;
	void previewInsert(QString tname, int col, int row, int alignment);
	void resizeEvent(QResizeEvent * event) override;

private:
	Ui::TablePreview *tp;
	QTableWidget *preview;
	TableDesignDialog *design;
	bool ok_to_close;
	bool preview_update_enabled;
	MainWindow *mw;
	balanceColumns *previewBalance;
	QTimer* resizeTimer;

Q_SIGNALS:
	void signal_keyToTable(int, int);

public Q_SLOTS:
	void slot_drawPreview();
	void slot_closePreview(bool save_settings);
	void slot_resizeTimeout();
	};

#endif // TABLEPREVIEW_H
