// This file is part of the disk cataloger Wssindex.  Wssindex is copyright
// 1985-2021 by Robert W. Babcock and WSS Division of DDC.  Effective 2010,
// Wssindex is distributed under the terms of the GNU General Public
// License.  Contact the author at wssddc@wssddc.com or see the project web
// page at <https://www.wssddc.com/wssindex/>.
//
// Wssindex uses the open source Qt toolkit from the Qt Project (qt-project.org)
// and The Qt Company (www.qt.io).
//
// Wssindex is free software: you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by the
// Free Software Foundation, either version 3 of the License, or (at your
// option) any later version.
//
// Wssindex is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
// for more details.
//
// You should have received a copy of the GNU General Public License along
// with Wssindex.  If not, see <https://www.gnu.org/licenses/>.

#include "OtherWssi/wssindex.h"
#include "OtherWssi/globalsettings.h"
#include "OtherWssi/wssiproto.h"
#include "OtherWssi/wsxxxadd.h"
#include "Dialogs/indexwindow.h"
#include "Dialogs/mainwindow.h"

#include <string>
#include <iostream>
#include <sstream>
#include <iomanip>

#include "MediaInfo/MediaInfo.h"
#include "ZenLib/Ztring.h"
#include <QDebug>

using namespace MediaInfoLib;
using namespace ZenLib;

int IndexWindow::mediaInfoSoundAdd(QString path, QString filename, int /* mode */)
	{
	if(skip_extract(filename))
		return(SKIPPED);

	int return_code = EXTRACTED_OK;

	MediaInfo MI;
	QString name_with_path = path;
	name_with_path += filename;

	if(!MI.Open(name_with_path.toStdWString()))
		return(CANT_OPEN);

	Ztring durationMsecStr = MI.Get(Stream_General, 0, __T("Duration"), Info_Text);
	Ztring bmode = MI.Get(Stream_Audio, 0, __T("BitRate_Mode"), Info_Text);
	bool variable_bitrate = __T("VBR") == bmode;
//	std::cout << "Bitmode = " << bmode.To_Local().c_str() << std::endl;

	Ztring Zbitrate = MI.Get(Stream_Audio, 0, __T("BitRate"), Info_Text);
//	std::cout << "Bitrate = " << Zbitrate.To_Local().c_str() << std::endl;
	int bitrate	=Zbitrate.To_int32s()/1000;

	int seconds = static_cast<int>(durationMsecStr.To_int64s()/1000);
	std::stringstream ss;
#define TEMPBUF_SIZE 256
	char tempbuf[TEMPBUF_SIZE];
	if(seconds >= 100)
		{
		int minutes = seconds/60;
		seconds -= 60*minutes;
		ss << bitrate << " kbps ";
		if(variable_bitrate)
			ss << "VBR ";
		ss << minutes <<":" << std::setfill('0') << std::setw(2) << seconds;
		strncpy_s(tempbuf, TEMPBUF_SIZE, ss.str().c_str(), TEMPBUF_SIZE-1);
		}
	else if(seconds > 0)
		{
		ss << bitrate << " kbps ";
		if(variable_bitrate)
			ss << "VBR ";
		ss << seconds << " sec";
		strncpy_s(tempbuf, TEMPBUF_SIZE, ss.str().c_str(), TEMPBUF_SIZE-1);
		}

	struct filerec *dr_nofile = Dr + Nofile - 1;
	if(extra_arch(tempbuf, dr_nofile, Max_extra_length))
		{
		return_code = DATABASE_FULL;
		Stop_indexing = true;
		QMessageBox::critical(this, tr("Error"), tr("Out of memory in mediaInfoSoundAdd"));
		}
	return(return_code);
	}
