// This file is part of the disk cataloger Wssindex.  Wssindex is copyright
// 1985-2021 by Robert W. Babcock and WSS Division of DDC.  Effective 2010,
// Wssindex is distributed under the terms of the GNU General Public
// License.  Contact the author at wssddc@wssddc.com or see the project web
// page at <https://www.wssddc.com/wssindex/>.
//
// Wssindex uses the open source Qt toolkit from the Qt Project (qt-project.org)
// and The Qt Company (www.qt.io).
//
// Wssindex is free software: you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by the
// Free Software Foundation, either version 3 of the License, or (at your
// option) any later version.
//
// Wssindex is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
// for more details.
//
// You should have received a copy of the GNU General Public License along
// with Wssindex.  If not, see <https://www.gnu.org/licenses/>.

#include "OtherWssi/wssindex.h"
#include "OtherWssi/wssiproto.h"
#include "OtherWssi/wsxxxadd.h"
#include "Dialogs/indexwindow.h"

// tgaadd - read a .TGA file and add information to wssindex database.
//	The image size and number of colors are converted into extra info
//	of the form WIDTHxHEIGHTxNCOLORS.
//
//	A TGA file with no image data will be reported as corrupt.

#pragma pack(1)

int IndexWindow::tgaadd(QString path, QString filename, int /* dummy */)
	{
	struct tga_header
		{
		char	id_length;
		char	map_type;
		char	image_type;
		short	map_origin;
		short	map_length;
		char	map_size;
		short	x_origin;
		short	y_origin;
		short	width;
		short	height;
		char	bits_per_pixel;
		char	image_descriptor;
		} tga;
	FILE *fp;
	unsigned long basefile;
	int	return_code;
	struct filerec *dr_nofile;

	if(skip_extract(filename))
		return(SKIPPED);

	basefile = Nofile - 1;
	return_code = EXTRACTED_OK;

	if(nullptr == (fp = arc_open(path, filename)))
		return(CANT_OPEN);

	if(1 != fread(&tga, sizeof(struct tga_header), 1, fp))
		{
		goto corrupt;			// this isn't an error, just a warning
		}

	dr_nofile = Dr + basefile;
	dr_nofile->ExtraInfo = tr("%1x%2x%3").arg(tga.width).arg(tga.height).arg(num_color_str(tga.bits_per_pixel));
	setHasExtra(dr_nofile);

#ifdef NOT_IMPLEMENTED
// Code to extract a comment would go here
	if(tga.id_length > 0)
		{
		if(1 != fread(Tempbuf, tga.id_length, 1, fp))
			goto corrupt;
		setHasComment(dr_nofile);
		}
#endif

norm_ret:
		fclose(fp);
		return(return_code);

corrupt:
		return_code = CORRUPT;
		Q_EMIT signal_logMessage(tr("%1%2 is corrupted or not a TGA image.")
				.arg(path, filename));
		goto norm_ret;
	}
