// This file is part of the disk cataloger Wssindex.  Wssindex is copyright
// 1985-2021 by Robert W. Babcock and WSS Division of DDC.  Effective 2010,
// Wssindex is distributed under the terms of the GNU General Public
// License.  Contact the author at wssddc@wssddc.com or see the project web
// page at <https://www.wssddc.com/wssindex/>.
//
// Wssindex uses the open source Qt toolkit from the Qt Project (qt-project.org)
// and The Qt Company (www.qt.io).
//
// Wssindex is free software: you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by the
// Free Software Foundation, either version 3 of the License, or (at your
// option) any later version.
//
// Wssindex is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
// for more details.
//
// You should have received a copy of the GNU General Public License along
// with Wssindex.  If not, see <https://www.gnu.org/licenses/>.

#include <stdio.h>
#include "OtherWssi/wssindex.h"
#include "OtherWssi/globalsettings.h"
#include "OtherWssi/wssiproto.h"
#include "Dialogs/indexwindow.h"
#include <QMessageBox>

// comarch - comment for archive file in buffer is trimmed to maximum allowed
//	length, filtered for non-printing characters, filtered for excess blanks,
//	and duplicated into dr_nofile->Comment.  Returns true on error, false
// on success.  (Actually, always returns false.)

bool IndexWindow::com_arch(char *buffer, struct filerec *dr_nofile, int length)
	{
	*(buffer+length) = EOS;
	makePrintable(buffer);
	*(buffer+Max_comment_length) = EOS;
	trimblnk(buffer);
	if(EOS == *buffer)			// anything left?
		return(false);
	if(!dr_nofile->Comment.isEmpty())
		{
// Are both possible comments the same?
		if(dr_nofile->Comment == buffer)
			return(false);

		if(!Always_diz_contents)
			{
			QMessageBox msgBox;
			if(inArc(dr_nofile))
				{
				struct filerec	*dr_arcbase = Dr + dr_nofile->Subdirno;
				msgBox.setText(tr("There are two possible comments for %1 in %2")
									.arg(dr_nofile->Lfn, dr_arcbase->Lfn));
				}
			else
				{
				msgBox.setText(tr("There are two possible comments for %1")
									.arg(dr_nofile->Lfn));
				}
			msgBox.setInformativeText(tr("1. %1\n2. %2").arg(dr_nofile->Comment.left(75),
											  QString::fromLatin1(buffer, qMin(static_cast<int>(strlen(buffer)),75))));
			QPushButton *use1Button = msgBox.addButton(tr("Use 1"), QMessageBox::ActionRole);
			msgBox.addButton(tr("Use 2"), QMessageBox::ActionRole);
			QPushButton *discardButton = msgBox.addButton(tr("Discard both"), QMessageBox::ActionRole);
			QPushButton *stopaskingButton = msgBox.addButton(tr("Stop asking"), QMessageBox::ActionRole);
			QPushButton *quitButton = msgBox.addButton(tr("Stop indexing"), QMessageBox::ActionRole);
			msgBox.exec();

			if (msgBox.clickedButton() == use1Button)
				return(false);
			if (msgBox.clickedButton() == discardButton)
				{
				dr_nofile->Comment = QString();
				dr_nofile->Diskbits &= ~HAS_COMMENT;
				return(false);
				}
			if (msgBox.clickedButton() == quitButton)
				{
				Stop_indexing = true;
				dr_nofile->Comment = QString();
				dr_nofile->Diskbits &= ~HAS_COMMENT;
				return(false);
				}
			if (msgBox.clickedButton() == stopaskingButton)
				Always_diz_contents = true;
			}
// Use 2 and stop asking fall through, as does Always_diz_contents==true
		}
	dr_nofile->Comment = buffer;
	setHasComment(dr_nofile);
	return(false);
	}
