// This file is part of the disk cataloger Wssindex.  Wssindex is copyright
// 1985-2021 by Robert W. Babcock and WSS Division of DDC.  Effective 2010,
// Wssindex is distributed under the terms of the GNU General Public
// License.  Contact the author at wssddc@wssddc.com or see the project web
// page at <https://www.wssddc.com/wssindex/>.
//
// Wssindex uses the open source Qt toolkit from the Qt Project (qt-project.org)
// and The Qt Company (www.qt.io).
//
// Wssindex is free software: you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by the
// Free Software Foundation, either version 3 of the License, or (at your
// option) any later version.
//
// Wssindex is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
// for more details.
//
// You should have received a copy of the GNU General Public License along
// with Wssindex.  If not, see <https://www.gnu.org/licenses/>.

#include <stdio.h>
#include "OtherWssi/wssindex.h"
#include "OtherWssi/globalsettings.h"
#include "Dialogs/indexwindow.h"
#include "OtherWssi/wssiproto.h"

// extra_arch - extra info for file in buffer is trimmed to maximum allowed
//	length, filtered for non-printing characters, filtered for excess blanks,
//	and duplicated into dr_nofile->Extrainfo.  Returns true on error, false
// on success.  (Actually, always returns false.)

//	This function is similar to com_arch, but simpler since it doesn't need
//	to worry about the possibility of extra info already having been assigned.

bool IndexWindow::extra_arch(char *buffer, struct filerec *dr_nofile, int length)
	{
	*(buffer+length) = EOS;
	makePrintable(buffer);
	*(buffer+Max_category_length) = EOS;
	trimblnk(buffer);
	if(EOS == *buffer)			// anything left?
		return(false);
	dr_nofile->ExtraInfo = buffer;
	setHasExtra(dr_nofile);
	return(false);
	}

bool IndexWindow::extra_arch(QString buffer, struct filerec *dr_nofile)
	{
//	makePrintable(buffer);
	buffer.truncate(Max_category_length);
	makePrintable(buffer);
	buffer = buffer.simplified();
	if(buffer.isEmpty())			// anything left?
		return(false);
	dr_nofile->ExtraInfo = buffer;
	setHasExtra(dr_nofile);
	return(false);
	}
