// This file is part of the disk cataloger Wssindex.  Wssindex is copyright
// 1985-2021 by Robert W. Babcock and WSS Division of DDC.  Effective 2010,
// Wssindex is distributed under the terms of the GNU General Public
// License.  Contact the author at wssddc@wssddc.com or see the project web
// page at <https://www.wssddc.com/wssindex/>.
//
// Wssindex uses the open source Qt toolkit from the Qt Project (qt-project.org)
// and The Qt Company (www.qt.io).
//
// Wssindex is free software: you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by the
// Free Software Foundation, either version 3 of the License, or (at your
// option) any later version.
//
// Wssindex is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
// for more details.
//
// You should have received a copy of the GNU General Public License along
// with Wssindex.  If not, see <https://www.gnu.org/licenses/>.

#include <QString>
#include "OtherWssi/wssindex.h"
#include "OtherWssi/tablelayout.h"

// Column and row are 1-based in settings, but have been changed to 0-based
// by the time this routine is called.
bool isTableDesignConsistent(struct fl *layout)
	{
	int celluse[MAXROWS][MAXCOLS];
	for(int i=0; i<MAXROWS; i++)
		for(int j=0; j<MAXCOLS; j++)
			celluse[i][j] = 0;

	for(int i=0; i<NFIELDS; i++)
		{
		if(!layout[i].enabled)
			continue;
		if(layout[i].width < 1)
			return false;
		if(layout[i].height < 1)
			return false;
		for(int k=layout[i].column; k<layout[i].column+layout[i].width; k++)
			{
			if(k<0 || k>=MAXCOLS)
				return false;
			for(int j=layout[i].row; j<layout[i].row+layout[i].height; j++)
				{
				if(j<0 || j>=MAXROWS)
					return false;
				celluse[j][k]++;
				if(celluse[j][k] > 1)
					return false;
				}
			}
		}
	return true;
	}
