// This file is part of the disk cataloger Wssindex.  Wssindex is copyright
// 1985-2021 by Robert W. Babcock and WSS Division of DDC.  Effective 2010,
// Wssindex is distributed under the terms of the GNU General Public
// License.  Contact the author at wssddc@wssddc.com or see the project web
// page at <https://www.wssddc.com/wssindex/>.
//
// Wssindex uses the open source Qt toolkit from the Qt Project (qt-project.org)
// and The Qt Company (www.qt.io).
//
// Wssindex is free software: you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by the
// Free Software Foundation, either version 3 of the License, or (at your
// option) any later version.
//
// Wssindex is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
// for more details.
//
// You should have received a copy of the GNU General Public License along
// with Wssindex.  If not, see <https://www.gnu.org/licenses/>.

#include "OtherWssi/wssindex.h"
#include "Threads/threadsafe.h"
#include "Dialogs/mainwindow.h"
#include "OtherWssi/globalsettings.h"

#include <stdlib.h>
#include <ctype.h>
#include <string.h>

// match - compare QStrings.  str1 may contain wildcard characters
//   * or ?  Returns true if args match, false if they do not.
//
// Comparison is case-insensitive.

bool match(const QString str1, const QString str2)
	{
	QRegExp rx(str1);
	rx.setCaseSensitivity(Qt::CaseInsensitive);
	rx.setPatternSyntax(QRegExp::Wildcard);
	return rx.exactMatch(str2);
	}

// Like match, but any diacritical marks or ligatures in str2
// are ignored.
bool matchNodiacritRegexp(const QRegExp rx, const QString str2, int ithread)
	{
// Normalization Form KD separates accents from characters
	QString nodia = str2.normalized(QString::NormalizationForm_KD);
// U+300 through U+36F is the Unicode Combining Diacritical Marks block
	nodia.replace(diamarksCopy[ithread], QFA(""));
// German eszett (looks like beta) is ligature for ss
	nodia.replace(eszettCopy[ithread], QFA("ss"));
	return rx.exactMatch(nodia);
	}

// matchLfn is similar to match except that the second string is the
//	file name in dr_k, no assumptions are made about the case of the input
//	strings.
bool matchLfn(const QString str1, const struct filerec *dr_k)
	{
	QString str2;

// Not equal if either input is null
	if(str1.isEmpty())
		return false;
	if(!dr_k)
		return false;

	str2 = dr_k->Lfn;
	QRegExp rx(str1);
	rx.setPatternSyntax(QRegExp::Wildcard);
	rx.setCaseSensitivity(Qt::CaseInsensitive);

	return rx.exactMatch(str2);
	}

// Like matchLfn, but the caller has constructed and saved the regular expression
bool matchLfnRegexp(const QRegExp rx, const struct filerec *dr_k)
	{
	if(!dr_k)
		return false;

	return rx.exactMatch(dr_k->Lfn);
	}

// Like match_lfn_nodiacrit, but the caller has constructed and saved the regular expression
bool matchLfnNodiacritRegexp(const QRegExp rx, const struct filerec *dr_k, int ithread)
	{
	QString str2;

// Not equal if 2nd name is null
	if(!dr_k)
		return false;

	str2 = dr_k->Lfn.normalized(QString::NormalizationForm_KD);
	str2.replace(diamarksCopy[ithread], QFA(""));
	str2.replace(eszettCopy[ithread], QFA("ss"));

	return rx.exactMatch(str2);
	}
