// This file is part of the disk cataloger Wssindex.  Wssindex is copyright
// 1985-2021 by Robert W. Babcock and WSS Division of DDC.  Effective 2010,
// Wssindex is distributed under the terms of the GNU General Public
// License.  Contact the author at wssddc@wssddc.com or see the project web
// page at <https://www.wssddc.com/wssindex/>.
//
// Wssindex uses the open source Qt toolkit from the Qt Project (qt-project.org)
// and The Qt Company (www.qt.io).
//
// Wssindex is free software: you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by the
// Free Software Foundation, either version 3 of the License, or (at your
// option) any later version.
//
// Wssindex is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
// for more details.
//
// You should have received a copy of the GNU General Public License along
// with Wssindex.  If not, see <https://www.gnu.org/licenses/>.

#include "Threads/binsearchthread.h"

BinSearchThread::BinSearchThread(int ithread_c, unsigned long *array_c, unsigned long *s_c,
	int *partnum_c, unsigned long *bincount_c, unsigned long low_c, unsigned long high_c,
	int (*cmpf_c)(const unsigned long, const unsigned long)) : QThread()
	{
	ithread = ithread_c;
	array = array_c;
	s = s_c;
	partnum = partnum_c;
	bincount = bincount_c;
	low = low_c;
	high = high_c;
	cmpf = cmpf_c;
	}

// Assign each element of array to a bin.  Bin boundaries are given by array s.
// If i is smallest value for which array[x] <= s[i], assign bin i.
// Otherwise bin is nthread-1.  Bin numbers go to partnum.  Counts in each
// bin go to bincount.  Range of subscripts to look at is low to high-1.
// compf is the comparison function.
void BinSearchThread::run()
	{
	for(unsigned long i=low; i<high; i++)
		{
		int imin = 0;
		int imax = ithread-1;
		while(imin < imax)
			{
			int imid = (imin+imax)/2;
			if(cmpf(s[imid], array[i]) < 0)
				imin = imid+1;
			else
				imax = imid;
			}
		partnum[i] = imin;
		bincount[imin]++;
		}
	}
